#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <sstream>
#include <string>

#include <QClipboard>
#include <QDate>
#include <QDir>
#include <QException>
#include <QFile>
#include <QFileDialog>
#include <QKeyEvent>
#include <QMessageBox>
#include <QSet>
#include <QString>
#include <QTextStream>

MainWindow::MainWindow(QWidget *parent)
	: QMainWindow(parent)
	, ui(new Ui::MainWindow)
	, settings("catsfood", "LaTeX-Creator")
{
	ui->setupUi(this);
	ui->date->setDate(QDate::currentDate());

	subjects = QSet<QString>{};

	QFile includesTex(includesTexPath);
	QFile definitionsTex(definitionsTexPath);
	QFile sectionTex(sectionTexPath);
	QFile documentHeaderTex(documentHeaderTexPath);
	QFile documentBodyPureTex(documentBodyPureTexPath);
	QFile documentBodyThemeTex(documentBodyThemeTexPath);

	if (includesTex.open(QFile::ReadOnly | QFile::Text))
	{
		QTextStream in(&includesTex);
		includesTexContent = in.readAll();
	}

	if (definitionsTex.open(QFile::ReadOnly | QFile::Text))
	{
		QTextStream in(&definitionsTex);
		definitionsTexContent = in.readAll();
	}

	if (sectionTex.open(QFile::ReadOnly | QFile::Text))
	{
		QTextStream in(&sectionTex);
		sectionTexContent = in.readAll();
	}

	if (documentHeaderTex.open(QFile::ReadOnly | QFile::Text))
	{
		QTextStream in(&documentHeaderTex);
		documentHeaderTexContent = in.readAll();
	}

	if (documentBodyPureTex.open(QFile::ReadOnly | QFile::Text))
	{
		QTextStream in(&documentBodyPureTex);
		documentBodyPureTexContent = in.readAll();
	}

	if (documentBodyThemeTex.open(QFile::ReadOnly | QFile::Text))
	{
		QTextStream in(&documentBodyThemeTex);
		documentBodyThemeTexContent = in.readAll();
	}
}

MainWindow::~MainWindow()
{
	delete ui;
}

std::ostream& operator <<(std::ostream &out, QString const &qstring)
{
	out << qstring.toStdString();
	return out;
}

void MainWindow::createClick()
{
	subjects.insert(ui->comboFach->currentText());

	std::stringstream content;

	//QString content = QString();
	content << includesTexContent << std::endl
			<< "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
			<< "\\newcommand{\\Fach}{" << ui->comboFach->currentText() << "}" << std::endl
			<< "\\newcommand{\\Name}{" << ui->txtName->text() << "}" << std::endl
			<< "\\newcommand{\\Uebung}{" << ui->udNumber->value() << "}" << std::endl
			<< "\\newcommand{\\Sheet}{" << ui->txtBezeichner->text() << "}" << std::endl
			<< "\\newcommand{\\Datum}{" << ui->date->date().toString("dd.MM.yyyy")<< "}" << std::endl
			<< "\\newcommand{\\Ort}{" << ui->txtOrt->text() << "}" << std::endl;

	if (ui->txtThema->text() != "")
	{
		content << "\\newcommand{\\Thema}{" << ui->txtThema->text() << "}"
				<< std::endl;
	}

	content << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl
			<< definitionsTexContent << std::endl;

	content << sectionTexContent << std::endl;

	if (ui->radioSubsectionYes->isChecked())
	{
		content << std::endl << createSubsectionHeading() << std::endl;
	}

	content << "\\makeatletter" << std::endl
			<< "\\renewcommand\\sectionlinesformat[4]{%" << std::endl
			<< "\t" << "\\Ifstr{#1}{section}{\\@hangfrom{\\hspace*{#2}#3}{";

	if (ui->radioTaskTitleYes->isChecked())
	{
		content << "\\normalfont\\sffamily\\small (#4)";
	}

	content << "}\\vspace*{-0.25\\baselineskip}}%" << std::endl
			<< "\t"
			<< "{%" << std::endl;

	if (ui->radioSubsectionYes->isChecked())
	{
		content << "\t\t"
				<< "\\Ifstr{#1}{subsection}{#3}%" << std::endl;
	}

	content << '\t'
			<< "}%" << std::endl
			<< "}" << std::endl
			<< "\\makeatother" << std::endl;

	content << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% section end %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" << std::endl;

	content << documentHeaderTexContent << std::endl;

	content << (ui->txtThema->text() == "" ? documentBodyPureTexContent : documentBodyThemeTexContent);

	if (ui->radioFileYes->isCheckable() && ui->txtFile->text() != "")
	{
		QFile outFile(ui->txtFile->text());
		if (outFile.open(QIODevice::ReadWrite))
		{
			QTextStream outStream(&outFile);
			outStream << QString::fromStdString(content.str());

			std::stringstream message;
			message << "Die Datei" << std::endl
					<< ui->txtFile->text().toStdString() << std::endl
					<< "wurde erfolgreich erzeugt.",

			QMessageBox::information(this,
									 "LaTeX-Creator",
									 QString::fromStdString(message.str()),
									 QMessageBox::Ok,
									 QMessageBox::Ok);
		}
	}
	else
	{
		QClipboard *clipboard = QGuiApplication::clipboard();
		clipboard->setText(QString::fromStdString(content.str()));

		QMessageBox::information(this,
								 "LaTeX-Creator",
								 "LaTeX-Code wurde in die Zwischenablage kopiert.",
								 QMessageBox::Ok,
								 QMessageBox::Ok);
	}

	settings.beginGroup("standard");
	settings.setValue("Ort", ui->txtOrt->text());
	settings.setValue("Name", ui->txtName->text());
	settings.endGroup();

	settings.beginWriteArray("subjects");
	int i = 0;
	for (auto &&subject : qAsConst(subjects))
	{
		settings.setArrayIndex(i);
		settings.setValue("subject", subject);
		++i;
	}
	settings.endArray();

	this->close();
}

void MainWindow::showEvent(QShowEvent *event)
{
	fileStateChanged();
	int size = settings.beginReadArray("subjects");
	for (int i = 0; i < size; ++i)
	{
		settings.setArrayIndex(i);
		QString text = settings.value("subject").toString();
		subjects.insert(text);
		ui->comboFach->addItem(text);
	}

	settings.endArray();

	settings.beginGroup("standard");
	ui->txtOrt->setText(settings.value("Ort").toString());
	ui->txtName->setText(settings.value("Name").toString());
	settings.endGroup();

	ui->comboFach->setCurrentText("");
	QMainWindow::showEvent(event);
}

void MainWindow::fileStateChanged()
{
	QString txtFileText = ui->radioFileNo->isChecked() ? "" : ui->txtFile->text();
	ui->txtFile->setText(txtFileText);

	ui->txtFile->setEnabled(ui->radioFileYes->isChecked());
	ui->btnFind->setEnabled(ui->radioFileYes->isChecked());
}

void MainWindow::findFileClick()
{
	QFileDialog dialog(this);
	dialog.setFileMode(QFileDialog::AnyFile);
	dialog.setNameFilter(tr("TeX-Dateien (*.tex)"));
	dialog.setViewMode(QFileDialog::Detail);

	if (dialog.exec())
	{
		ui->txtFile->setText(dialog.selectedFiles().at(0));
	}
}

std::string MainWindow::createSubsectionHeading()
{
	std::stringstream content;
	content << "\\RedeclareSectionCommands[runin=true,beforeskip=0.3\\baselineskip]{subsection}"
			<< std::endl;

	QString numbering;
	switch (ui->comboSubsectionCounter->currentIndex())
	{
		case 0:
			numbering = "\\arabic{subsection}";
			break;
		case 1:
			numbering = "\\roman{subsection}";
			break;
		case 2:
			numbering = "\\Roman{subsection}";
			break;
		case 3:
			numbering = "\\Alph{subsection}";
			break;
		case 4:
			numbering = "\\alph{subsection}";
			break;
		default:
			throw QException();
			break;
	}
	content << "\\renewcommand*\\thesubsection{(" << numbering << ")}" << std::endl
			<< "\\let\\oldsubsection\\subsection" << std::endl
			<< "\\renewcommand{\\subsection}{\\oldsubsection[Teilaufgabe " << numbering << "]{}}" << std::endl;

	return content.str();
}

void MainWindow::radioSubsectionCheckedChanged()
{
	bool enabled = ui->radioSubsectionYes->isChecked();
	ui->comboSubsectionCounter->setEnabled(enabled);
	ui->txtAfterSubsecCounter->setEnabled(enabled);
	ui->txtBeforeSubsecCounter->setEnabled(enabled);
}

void MainWindow::keyPressEvent(QKeyEvent *event)
{
	switch (event->key())
	{
		case Qt::Key_Escape:
			this->close();
			break;
		default:
			break;
	}
}
